<?php
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)

$startLoadTime = microtime(true);
set_time_limit(10);

// This function is to be moved into UNB's future i18n support.
function format_ordinal($num, $html = true)
{
	if ($num <= 0) return $num;
	if ($num % 10 == 1) return $num . ($html ? '<sup>' : '') . 'st' . ($html ? '</sup>' : '');
	if ($num % 10 == 2) return $num . ($html ? '<sup>' : '') . 'nd' . ($html ? '</sup>' : '');
	if ($num % 10 == 3) return $num . ($html ? '<sup>' : '') . 'rd' . ($html ? '</sup>' : '');
	return $num . ($html ? '<sup>' : '') . 'th' . ($html ? '</sup>' : '');
}

function format_filesize($size)
{
	if ($size < 0.9 * 1024) return $size . ' bytes';
	if ($size < 50 * 1024) return round($size / 1024, 1) . ' KiB';
	if ($size < 0.9 * 1024 * 1024) return round($size / 1024) . ' KiB';
	if ($size < 50 * 1024 * 1024) return round($size / 1024 / 1024, 1) . ' MiB';
	if ($size < 0.9 * 1024 * 1024 * 1024) return round($size / 1024 / 1024) . ' MiB';
	if ($size < 50 * 1024 * 1024 * 1024) return round($size / 1024 / 1024 / 1024, 1) . ' GiB';
	return round($size / 1024 / 1024 / 1024) . ' GiB';
}

function mimetype_description($type)
{
	switch ($type)
	{
		case 'application/octet-stream': return 'Unspecified binary file';
		case 'text/plain': return 'Plain text';
		case 'image/png': return 'PNG image';
		case 'image/gif': return 'GIF image';
		case 'image/jpeg': return 'JPEG image';
		case 'application/vnd.ms-excel': return 'Microsoft Excel document';
		case 'application/x-pdf': return 'Adobe PDF document';
		case 'application/x-zip-compressed': return 'ZIP archive';
	}
	return $type;
}

function mimetype_icon($type)
{
	$type = str_replace('/', '_', $type);
	$type = str_replace('-', '_', $type);
	if (file_exists('img/mimetype/' . $type . '.png'))
		return 'img/mimetype/' . $type . '.png';
	return 'img/mimetype/unknown.png';
}

$libPath = '../unb_lib/';
#require_once($libPath . 'unb_all.lib.php');
if (!class_exists('UnbMarkup')) require_once($libPath . 'markup.class.php');
if (!class_exists('UnbCaptcha')) require_once($libPath . 'captcha.class.php');
if (!class_exists('UnbUi')) require_once($libPath . 'ui.class.php');

// Make sure output buffering is active.
UnbEnvironment::SetupEnvironment();
// Re-enable error display in testing environment.
ini_restore('display_errors');

$startExecTime = microtime(true);

if (!defined('NOHTML'))
{
	header('Content-Type: text/html; charset=utf-8');
	?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html>
<head>
<title>UNB2 Demo Application</title>
<link rel="stylesheet" type="text/css" href="common.css" />
<link rel="stylesheet" type="text/css" href="markup.css" />
<!--[if IE]><link rel="stylesheet" type="text/css" href="common_ie.css" /><![endif]-->
<script src="dynamic-complete.js" type="text/javascript"></script>
</head>
<body>

<?php

/*if ($_POST)
{
	echo '<pre>';
	print_r($_POST);
	echo '</pre>';
}*/

} // if (!defined('NOHTML'))

UnbDebug::StartErrorHandling(true);

// Now that the HTML page is ready and error handling is set up, initialise the rest.
// The settings file name is relative to the UNB library source path.
UNB::Init('../unb_config.inc.php');

if (!defined('NOHTML'))
{
?>
<div class="head">
<div class="logo">
UNB2 Demo Application
</div>

<div style="float: right;">
<?php
$sessionUser = UnbSession::GetSessionUser();
if (isset($sessionUser))
{
	echo 'Logged in as: <strong><a href="alter_user.php?userid=' . $sessionUser->GetUserId() . '">' .
		htmlspecialchars($sessionUser->GetDisplayName()) .
		'</a></strong>';
	$statusStr = '';
	if ($sessionUser->IsAdministrator())
		$statusStr .= ($statusStr ? ', ' : '') . 'Admin';
	if ($sessionUser->IsModerator())
		$statusStr .= ($statusStr ? ', ' : '') . 'Mod';
	if ($sessionUser->IsTrusted())
		$statusStr .= ($statusStr ? ', ' : '') . 'Trusted';
	if ($statusStr)
		echo ' <small>(' . $statusStr . ')</small>';

	if (!strlen($sessionUser->GetLoginName()) ||
		!strlen($sessionUser->GetLoginPassword()))
	{
		echo ' &nbsp;–&nbsp; <a href="logout_warning.php">Log out (warning)</a>';
	}
	else
	{
		echo ' &nbsp;–&nbsp; <a href="login.php?action=logout&amp;urlkey=' . UnbSession::GetUrlKey() . '">Log out</a>';
	}
}
else
{
	echo 'Not logged in';
	echo ' &nbsp;–&nbsp; <a href="login.php">Log in or register</a>';
}
?>
</div>

<div>
<a href="index.php">Start page</a> &nbsp;–&nbsp;
<a href="list_deps.php">Discussions</a> &nbsp;–&nbsp;
<a href="list_tags.php">Tags</a> &nbsp;–&nbsp;
<a href="list_users.php">Users</a>
</div>
<div style="clear: both;"></div>
</div>

<?php
} // if (!defined('NOHTML'))
?>