<?php
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
require('_head.inc.php');

// ----- Handle form requests and status messages -----
if (UnbUi::IsFormSuccess())
{
	UnbSession::SetVar('SuccessMessage', 'Tag saved');
	UnbUi::RedirectAndExit('list_tags.php');
}
if (UnbUi::IsFormError())
{
	UnbUi::ReportError(UnbUi::GetFormErrorMessage());
}
if ($m = UnbSession::GetVar('SuccessMessage', true))
{
	UnbUi::ReportSuccess($m);
}

// ----- Load the tag -----
$tag = UnbTag::FindByTagId(intval($_REQUEST['tagid']));
if (!UnbSession::IsLoggedIn())
{
	UnbUi::ReportError('Not logged in');
}
else if (!isset($tag))
{
	UnbUi::ReportError('Invalid tag ID');
}
else
{
	?>
	<h1>Alter tag “<?php echo htmlspecialchars($tag->GetTagName()) ?>”</h1>

	<?php UnbUi::BeginForm('alter_tag.php') ?>
	<?php UnbUi::FormTag($tag->GetTagId()) ?>

	<table cellspacing="0" class="grid first-10em">
	<tr>
		<td>Tag name:</td>
		<?php
		if (UnbSession::IsModerator())
		{
			?>
			<td><?php UnbUi::TagName($tag) ?></td>
			<td class="form-description">This is the visible tag name.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2"><?php echo htmlspecialchars($tag->GetTagName()); ?></td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td>Description:</td>
		<?php
		if (UnbSession::IsModerator())
		{
			?>
			<td><?php UnbUi::TagDescription($tag) ?></td>
			<td class="form-description">The description of this tag explains the user when and why to use it.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2"><?php echo htmlspecialchars($tag->GetDescription()); ?></td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td>Flags:</td>
		<?php
		if (UnbSession::IsModerator())
		{
			?>
			<td><?php UnbUi::TagFlags($tag) ?></td>
			<td class="form-description">Flags set special tag behaviour options.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2">
				<?php
				$names = array();
				if ($tag->GetFlags() & UnbTagFlag::NotSelectable) $names[] = 'Not selectable';
				if ($tag->GetFlags() & UnbTagFlag::Invisible) $names[] = 'Invisible';
				if ($tag->GetFlags() & UnbTagFlag::EnforceApproval) $names[] = 'Enforce approval';
				if ($tag->GetFlags() & UnbTagFlag::UserData) $names[] = 'With user data';
				if ($names)
					echo join(', ', $names);
				else
					echo '<em>(None)</em>';
				?>
			</td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td>Parent tag:</td>
		<?php
		if (UnbSession::IsModerator())
		{
			?>
			<td><?php UnbUi::ParentTag($tag) ?></td>
			<td class="form-description">A tag’s parent tag is used to organise tags in a tree hierarchy. Invisible or user data tags cannot be used as a parent.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2">
				<?php
				$parentTag = $tag->GetParentTag();
				if ($parentTag)
					echo htmlspecialchars($parentTag->GetTagName());
				else
					echo '<em>(None)</em>';
				?>
			</td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td>Read access for:</td>
		<?php
		if (UnbSession::IsAdministrator())
		{
			?>
			<td><?php UnbUi::TagReadAccessList($tag, null, '', 'Everybody – click here to specify users') ?></td>
			<td class="form-description">List of user names or keys that can read messages with this tag, one per line. Unrestricted if empty.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2">
				<?php
				$readAccessList = UnbUi::GetTagReadAccessKeys($tag);
				if (strlen($readAccessList))
					echo nl2br(htmlspecialchars($readAccessList));
				else
					echo '<em>(Not restricted)</em>';
				?>
			</td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td>Usage access for:</td>
		<?php
		if (UnbSession::IsAdministrator())
		{
			?>
			<td><?php UnbUi::TagUseAccessList($tag, null, '', 'Everybody – click here to specify users') ?></td>
			<td class="form-description">List of user names or keys that can assign this tag to messages, one per line. Unrestricted if empty.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2">
				<?php
				$useAccessList = UnbUi::GetTagUseAccessKeys($tag);
				if (strlen($useAccessList))
					echo nl2br(htmlspecialchars($useAccessList));
				else
					echo '<em>(Not restricted)</em>';
				?>
			</td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td></td>
		<td colspan="2">
			<br />
			<input type="submit" class="default icon13 ok" value="Save changes" /> &nbsp;
			<a href="list_tags.php" class="buttonlike icon13 back">Back to tags list</a>
			<?php
			if (UnbSession::IsModerator())
			{
				?>
				&nbsp; <a href="delete_tag.php?tagid=<?php echo $tag->GetTagId() ?>" class="buttonlike icon13 delete">Delete this tag…</a>
				<?php
			}
			?>
		</td>
	</tr>
	</table>
	<?php UnbUi::EndForm() ?>

	<script type="text/javascript">//<![CDATA[
	$("#tagNameTb").focus();
	//]]></script>

	<?php
}

require('_foot.inc.php');
?>