<?php
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
require('_head.inc.php');

if ($m = UnbSession::GetVar('SuccessMessage', true))
{
	UnbUi::ReportSuccess($m);
}
?>

<h1>Discussions list</h1>

<?php
$drafts = UnbMessageRevision::FindAllDrafts();

if (count($drafts))
{
	echo '<p style="margin-bottom: 2em;">';
	echo '<img src="img/btn-draft.png" alt="" style="margin-bottom: -2px; margin-right: 7px;" />';
	echo 'You have saved ' . count($drafts) . ' drafts:<br />';

	foreach ($drafts as $draftRevId)
	{
		$rev = UnbMessageRevision::FindByRevisionId($draftRevId, UnbMessageRevisionContentLevel::Subject);

		echo '<a href="create_message.php?drarid=' . $draftRevId . '">' . htmlspecialchars($rev->GetSubject()) . '</a>';
		echo ' &nbsp; Last saved <span class="date" title="' . UnbUi::DateTime($rev->GetCreatedTime(), true, true) . '">' . UnbUi::RelativeTime($rev->GetCreatedTime()) . '</span>';
		echo '<br />';
	}
	echo '</p>';
}

try
{
	$searchParameters = array();
	$searchParameters['entrypoint'] = true;
	$searchParameters['sort-createdtime'] = true;
	$searchParameters['sort-reverse'] = true;
	$messageIds = UnbSearch::Query($searchParameters);

	echo '<p>' . count($messageIds) . ' discussions in this list';
	echo ' &nbsp;–&nbsp; <a href="create_message.php" class="buttonlike icon13 create">Create a new message</a>';
	if (UnbSession::IsLoggedIn())
		echo ' <a href="create_file.php" class="buttonlike icon13 create">Create a new file</a>';
	echo '</p>';

	$markup = new UnbMarkup();
	$discussionCount = 0;
	if ($messageIds) foreach ($messageIds as $messageId)
	{
		$discussionCount++;

		$msg = UnbMessage::FindByMessageId($messageId);
		if (!isset($msg)) continue;   // Actually, this should not happen here anymore

		$searchRev = $msg->GetSearchRevision(UnbMessageRevisionContentLevel::Subject, true);
		$latestRev = $msg->GetLatestRevision(UnbMessageRevisionContentLevel::Subject, true);
		if (!isset($searchRev)) continue;   // Actually, this should not happen here anymore
		if (!isset($latestRev)) continue;   // Actually, this should not happen here anymore

		echo '<div class="discussion' . ($discussionCount == 1 ? ' first' : '') . '">';
		echo '<div class="subject-line">';
		echo '<span class="subject">';
		if ($searchRev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject)
		{
			if (strlen($searchRev->GetSubject()))
				$text = htmlspecialchars($searchRev->GetSubject());
			else
				$text = '(No subject set)';

			echo '<a href="view_message.php?messageid=' . $msg->GetMessageId() . '">' . $text . '</a>';
		}
		else
		{
			echo '<em>No subject accessible</em>';
		}
		echo '</span>';   // .subject

		$tags = $searchRev->GetTags(true);
		if ($tags)
		{
			echo '<span class="tags">';
			$count = 0;
			foreach ($tags as $tag)
			{
				#if ($count++) echo ', ';
				echo '<span class="tag" title="' . htmlspecialchars($tag->GetDescription()) . '">';
				echo $tag->GetTagName();
				if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
					echo ': ' . htmlspecialchars($tag->GetTagData());
				echo '</span> ';
			}
			echo '</span>';   // .tags
		}
		echo '</div>';   // .subject-line

		$flags = $msg->GetModerationFlags();

		echo '<div class="second-line">';
		echo '<span class="author">';
		echo 'by <span class="name">' . $msg->GetOwnerUser()->GetDisplayName() . '</span>';
		echo '</span>';

		$msgCreatedTime = $msg->GetCreatedTime();
		echo '<span class="date" title="' . UnbUi::DateTime($msgCreatedTime, true, true) . '">' . UnbUi::RelativeTime($msgCreatedTime) . '</span>';
		$latestReply = $msg->GetLatestReply();
		if (isset($latestReply))
		{
			$repliesCount = $msg->CountReplies();
			$latestReplyTime = $latestReply->GetCreatedTime();
			echo '<span class="replies">';
			if ($repliesCount == 1)
				echo '<strong>' . $repliesCount . '</strong> reply, <span title="' . UnbUi::DateTime($latestReplyTime, true, true) . '">' . UnbUi::RelativeTime($latestReplyTime) . '</span>';
			else
				echo '<strong>' . $repliesCount . '</strong> replies, latest <span title="' . UnbUi::DateTime($latestReplyTime, true, true) . '">' . UnbUi::RelativeTime($latestReplyTime) . '</span>';
			echo '</span>';   // .replies
		}

		$rating = $msg->GetRating();
		if ($rating[1] >= /*30*/ 1)   // TODO: Reset for release
		{
			UnbUi::MsgRating($msg, 1, 30, '|msg-rating');
		}

		echo '</div>';   // .second-line
		echo '</div>';   // .discussion

		echo '<div class="attachinfoline">';
		if ($msg->HasFileData())
		{
			$size = $searchRev->GetFileDataLength() . ' bytes';

			echo 'There is additional data attached:
				<a href="get_data.php?messageid=' . $messageId . '&amp;messagerevisionid=' . $searchRev->GetMessageRevisionId() . '">Download</a> (' . $size . ')<br />';
		}
		echo '</div>';

		echo '<div class="modwarning">';
		if ($flags & UnbMessageModerationFlag::Hidden)
		{
			echo 'This message has been hidden by a moderator. &nbsp; ';
		}
		if ($flags & UnbMessageModerationFlag::Locked)
		{
			echo 'This message has been locked by a moderator. &nbsp; ';
		}
		// If searchRevision != latestRevision, it must be because of a moderation state which is covered below:
		if ($latestRev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
		{
			echo 'The latest message revision is waiting for approval. &nbsp; ';
		}
		else if ($latestRev->GetModerationState() == UnbMessageRevisionModerationState::Locked)
		{
			echo 'The latest message revision has been locked by a moderator. &nbsp; ';
		}
		echo '</div>';
		echo '</div>';
	}
}
catch (UnbSecurityException $ex)
{
	UnbUi::ReportError($ex);
}
?>

<?php
require('_foot.inc.php');
?>