<?php
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
require('_head.inc.php');

// ----- Handle form requests and status messages -----
if (UnbUi::IsFormSuccess())
{
	UnbUi::RedirectAndExit('index.php');
}
if (UnbUi::IsFormError())
{
	UnbUi::ReportError(UnbUi::GetFormErrorMessage());
}
if ($m = UnbSession::GetVar('SuccessMessage', true))
{
	UnbUi::ReportSuccess($m);
}

$messageId = intval($_REQUEST['messageid']);

$msg = UnbMessage::FindByMessageId($messageId);
if (!isset($msg))
{
	UnbUi::ReportError('Invalid message ID');
}
else
{
	echo '<h1>Message revisions list</h1>';

	$revs = $msg->GetAllRevisions(true);
	if (!isset($revs))
	{
		UnbUi::ReportError('Internal error: No revisions found');
	}
	else
	{
		$searchRev = $msg->GetSearchRevision(UnbMessageRevisionContentLevel::NoContent);

		$prevTags = null;

		foreach ($revs as $rev)
		{
			$sr = '';
			if ($rev->GetMessageRevisionId() == $searchRev->GetMessageRevisionId()) $sr = ' <strong>(Public revision)</strong>';

			echo '<a href="view_message.php?messagerevisionid=' . $rev->GetMessageRevisionId() . '&amp;alone=1">Revision ' . $rev->GetRevisionNumber() . '</a> ' .
				'by <strong>' . $rev->GetAuthorUser()->GetDisplayName() . '</strong>, ' .
				'<span title="' . UnbUi::DateTime($rev->GetCreatedTime(), true, true) . '">' . UnbUi::RelativeTime($rev->GetCreatedTime()) . '</span>' .
				$sr;

			echo ' &nbsp;–&nbsp; ';
			if ($rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject)
			{
				if (strlen($rev->GetSummary()))
					echo htmlspecialchars($rev->GetSummary());
				else if ($rev->GetRevisionNumber() == 1)
					echo '<em>First revision</em>';
				else
					echo '<em>No summary available</em>';
			}
			else
			{
				echo '<em>No summary accessible</em>';
			}

			$tags = $rev->GetTags(true);
			if (!isset($prevTags))
			{
				// First revision: Show all tags
				if ($tags)
				{
					echo ' &nbsp; ';
					foreach ($tags as $tag)
					{
						echo '<span class="tag">';
						echo $tag->GetTagName();
						if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
							echo ': ' . htmlspecialchars($tag->GetTagData());
						echo '</span> ';
					}
				}
			}
			else
			{
				// Second or later revision: Show tag differences
				UnbTag::GetDifferences($prevTags, $tags, $addedTags, $removedTags, $changedTags);
				$str = '';
				foreach ($addedTags as $tag)
				{
					$str .= '<span class="tag added">+';
					$str .= $tag->GetTagName();
					if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
						$str .= ': ' . htmlspecialchars($tag->GetTagData());
					$str .= '</span> ';
				}
				foreach ($changedTags as $tag)
				{
					$str .= '<span class="tag changed">*';
					$str .= $tag->GetTagName();
					if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
						$str .= ': ' . htmlspecialchars($tag->GetTagData());
					$str .= '</span> ';
				}
				foreach ($removedTags as $tag)
				{
					$str .= '<span class="tag removed">&minus;';
					$str .= $tag->GetTagName();
					if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
						$str .= ': ' . htmlspecialchars($tag->GetTagData());
					$str .= '</span> ';
				}
				if (strlen($str))
				{
					echo ' &nbsp; ' . $str;
				}
			}
			$prevTags = $tags;

			if ($rev->IsDraft())
			{
				echo ' &nbsp; <span class="modwarning">This is a draft.</span>';
			}

			if ($rev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
			{
				echo ' &nbsp; <span class="modwarning">This message revision is waiting for approval.</span>';
			}
			else if ($rev->GetModerationState() == UnbMessageRevisionModerationState::Locked)
			{
				echo ' &nbsp; <span class="modwarning">This message revision has been locked by a moderator.</span>';
			}

			echo '<br />';
		}

		echo '<br /><a href="view_message.php?messageid=' . $messageId . '#message-' . $messageId . '" class="buttonlike icon13 back">Back to message view</a>';
	}
}

require('_foot.inc.php');
?>