<?php
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
require('_head.inc.php');

$messageId = isset($_REQUEST['messageid']) ? intval($_REQUEST['messageid']) : null;
$messageRevisionId = isset($_REQUEST['messagerevisionid']) ? intval($_REQUEST['messagerevisionid']) : null;
$rootMessageId = isset($_REQUEST['root']) ? intval($_REQUEST['root']) : null;
$returnToMessageId = isset($_REQUEST['retmid']) ? intval($_REQUEST['retmid']) : null;

// ----- Handle form requests and status messages -----
if (UnbUi::IsFormSuccess())
{
	$reqMessageId = null;
	if (UnbUi::GetFormHandlerFields('moderate_messagerevision', 'success') !== null)
	{
		if ($_REQUEST['message_modstate'] == UnbMessageRevisionModerationState::Approved)
			UnbSession::SetVar('SuccessMessage', 'Message revision approved/unlocked');
		if ($_REQUEST['message_modstate'] == UnbMessageRevisionModerationState::Locked)
			UnbSession::SetVar('SuccessMessage', 'Message revision locked');
		$reqMessageId = UnbUi::GetFormHandler()->GetRequestedMessageRevision()->GetMessageId();
	}

	if (isset($returnToMessageId))
	{
		$messageId = $returnToMessageId;
		$messageRevisionId = null;
	}

	$params = array();
	$hash = null;
	if (isset($messageId))
	{
		$params['messageid'] = $messageId;
		$hash = 'message-' . $messageId;
	}
	if (isset($messageRevisionId))
	{
		$params['messagerevisionid'] = $messageRevisionId;
		if ($reqMessageId)
		{
			$hash = 'message-' . $reqMessageId;
		}
	}
	UnbUi::RedirectAndExit('view_message.php', $params, $hash);
}

UnbUi::BeginReport();
if (UnbUi::IsFormError())
{
	UnbUi::ReportError(UnbUi::GetFormErrorMessage());
}
if ($m = UnbSession::GetVar('SuccessMessage', true))
{
	UnbUi::ReportSuccess($m);
}

// ----- Load the message revision -----
$error = false;
if (!$messageId && !$messageRevisionId)
{
	UnbUi::ReportError('No message selected.');
	$error = true;
}
else if (!UnbUi::GetMsgRev($messageId, $messageRevisionId, $msg, $rev, $err, true))
{
	UnbUi::ReportError($err);
	$error = true;
}
UnbUi::EndReport();

if (!$error)
{
	$markup = new UnbMarkup();
	$showAvatars = false;

	if ($_REQUEST['alone'] || $msg->HasFileData())
	{
		echo '<div class="msg-view' . ($showAvatars ? ' with-avatars' : '') . '">';
		show_message($msg, $rev);
		echo '</div>';   // .msg-view
	}
	else
	{
		$prevTags = null;
		$prevSubject = null;

		$depId = $msg->GetRefDEP();
		if ($depId != $msg->GetMessageId())
		{
			$depMsg = UnbMessage::FindByMessageId($depId);
			$depRev = $depMsg->GetSearchRevision(UnbMessageRevisionContentLevel::Subject, true);
		}
		else
		{
			$depMsg =& $msg;
			$depRev =& $rev;
		}
		echo '<h1 style="margin-bottom: 0.3em;">' . htmlspecialchars($depRev->GetSubject()) . '</h1>';
		$prevSubject = $depRev->GetSubject();

		$tags = $depRev->GetTags(true);
		$prevTags = $tags;
		if ($tags)
		{
			$count = 0;
			foreach ($tags as $tag)
			{
				#if ($count++) echo ', ';
				echo '<span class="tag" title="' . htmlspecialchars($tag->GetDescription()) . '">';
				echo $tag->GetTagName();
				if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
					echo ': ' . htmlspecialchars($tag->GetTagData());
				echo '</span> ';
			}
		}

		echo '<div style="margin: 1em 0; border-top: solid 1px #c0c0c0;"></div>';

		echo '<table style="clear: both; width: 100%; table-layout: fixed;" cellspacing="0">';
		echo '<tr>';
		echo '<td style="padding: 0; vertical-align: top; width: 17em; padding-right: 2em;">';

		// Maximum level of replies to show on one page
		$maxLevel = 7;

		if (!isset($rootMessageId))
		{
			// No message is specified as tree root.
			// Test whether the specified message is visible in the message tree.
			$depId2 = $msg->GetRefDEP(intval($maxLevel / 2));
			if ($depId2 != $depId)
			{
				// A limited DEP search has returned a different root message.
				// Use that as tree root to ensure that the specified message is well visible on this page.
				$rootMessageId = $depId2;
			}
		}

		echo 'Messages in this discussion:';
		echo '<div class="msg-tree">';
		UnbUi::MsgTree($msg->GetMessageId(), $rootMessageId, true, $maxLevel, '<a href="view_message.php?%q' . (isset($rootMessageId) ? '&root=' . $rootMessageId : '') . '#message-%i">%s</a>', 'view_message.php?messageid=%i&root=%r');
		echo '</div>';

		echo '</td>';
		echo '<td style="padding: 0; vertical-align: top;">';

		echo '<div class="msg-view' . ($showAvatars ? ' with-avatars' : '') . '">';

		$count = 0;
		$items = UnbUi::GetTreeMessages();
		foreach ($items as &$item)
		{
			if ($item['selected'])
			{
				if ($count++ > 0)
				{
					echo '<div class="message-separator"></div>';
				}

				$replies = array();
				foreach ($items as &$item2)
				{
					if ($item2['msg']->GetPrimaryReference() == $item['msg']->GetPrimaryReference())
					{
						$replies[] = $item2;
					}
				}
				if (count($replies) > 1)
				{
					echo '<div class="message notice">';
					echo '<div class="content" style="margin-bottom: 0.2em;">';
					echo '<img src="img/multiple-replies.png" alt="" /> ';
					echo 'Multiple replies to this message, select one to display:';
					echo '</div>';   // .content
					echo '</div>';   // .message-nogap

					foreach ($replies as $reply)
					{
						$msg = $reply['msg'];
						$rev = $reply['rev'];
						$msgCreatedTime = $msg->GetCreatedTime();

						$counting = false;
						$minLevel = 0;
						$count = 0;
						foreach ($items as &$item2)
						{
							if ($item2['msg']->GetMessageId() == $msg->GetMessageId())
							{
								$counting = true;
								$minLevel = $item2['level'];
								$count++;
								if ($item2['morecount'])
									$count += $item2['morecount'];
							}
							else if ($counting && $item2['level'] == $minLevel)
							{
								$counting = false;
							}
							else if ($counting)
							{
								$count++;
								if ($item2['morecount'])
									$count += $item2['morecount'];
							}
						}

						echo '<div class="message notice">';
						if ($showAvatars)
						{
							echo '<div class="avatarcontainer">';
							echo '<img src="img/avatar_' . $msg->GetOwnerUser()->GetLoginName() . '.jpg" style="max-width: 50%; max-height: 1.2em;" />';
							echo '</div>';   // .avatarcontainer
						}

						echo '<div class="content">';
						if (!$reply['selected'])
							echo '<a href="view_message.php?messageid=' . $msg->GetMessageId() . (isset($rootMessageId) ? '&root=' . $rootMessageId : '') . '#message-' . $msg->GetMessageId() . '">';
						echo '<strong>' . htmlspecialchars($msg->GetOwnerUser()->GetDisplayName()) . '</strong>';
						if (!$reply['selected'])
							echo '</a>';

						if ($rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject &&
							strlen($rev->GetSubject()) &&
							$rev->GetSubject() != $prevSubject &&
							$rev->GetSubject() != 'Re: ' . $prevSubject)
						{
							echo ' &nbsp; “' . htmlspecialchars($rev->GetSubject()) . '”';
						}
						echo ' &nbsp;–&nbsp; <span title="' . UnbUi::DateTime($msgCreatedTime, true, true) . '">' . UnbUi::RelativeTime($msgCreatedTime) . '</span>';
						echo ' &nbsp;–&nbsp; ' . $count . ' replies altogether';
						if ($reply['selected'])
							echo ' &nbsp; (displayed below)';
						if ($rev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
							echo ' &nbsp;–&nbsp; <span class="modwarning">Waiting for approval</span>';
						echo '<br />';
						echo '</div>';   // .content
						echo '</div>';   // .message-nogap
					}

					echo '<div class="message-separator"></div>';
				}

				$showMsg = $item['msg'];
				$showRev = $item['rev'];
				if ($rev->GetMessageId() == $showMsg->GetMessageId() &&
					$rev->GetMessageRevisionId() != $showRev->GetMessageRevisionId())
				{
					// Another revision of this message was explicitly specified, so display that instead of the default revision
					$showRev = $rev;
				}
				show_message($showMsg, $showRev);
			}
		}

		echo '</div>';   // .msg-view
		echo '</td>';
		echo '</tr>';
		echo '</table>';
	}
}
?>

<script>
function startHighlight(id)
{
	DOM.addClass(id, "highlight");
	setTimeout(function() { DOM.addClass(id, "highlight-fade"); }, 5000);
	// Clean up
	setTimeout(function() { DOM.removeClass(id, "highlight highlight-fade"); }, 7000);
}

function historyChanged()
{
	if (window.location.hash.match(/^#message-[0-9]+$/))
	{
		startHighlight(window.location.hash + "-div");
	}
}

historyChanged();

DOM.addHistoryChangedHandler(historyChanged);
</script>

<?php
require('_foot.inc.php');


function show_message($msg, $rev)
{
	global $markup, $prevTags, $prevSubject, $showAvatars;

	$isFile = $msg->HasFileData();

	echo '<a id="message-' . $msg->GetMessageId() . '" style="position: relative; top: -2em;"></a>';
	echo '<div class="message" id="message-' . $msg->GetMessageId() . '-div">';

	if ($showAvatars)
	{
		echo '<div class="avatarcontainer">';
		echo '<img src="img/avatar_' . $msg->GetOwnerUser()->GetLoginName() . '.jpg" style="max-width: 100%; max-height: 5em;" />';
		echo '</div>';   // .avatarcontainer
	}

	echo '<div class="sideactions">';
	echo '<div>';
	if ($msg->GetOwnerUser()->IsAdministrator() && $msg->GetOwnerUser()->IsModerator())
		echo 'Author is an administrator and a moderator.<br />';
	else if ($msg->GetOwnerUser()->IsAdministrator())
		echo 'Author is an administrator.<br />';
	else if ($msg->GetOwnerUser()->IsModerator())
		echo 'Author is a moderator.<br />';
	if (!$isFile &&
		$msg->GetSearchRevisionId() == $rev->GetMessageRevisionId() &&
		$rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Content &&
		$msg->IsReplyAllowed())
		echo '<strong><a href="create_message.php?reprid=' . $rev->GetMessageRevisionId() . '">Reply to this message</a></strong><br />';
	if ($msg->GetSearchRevisionId() == $rev->GetMessageRevisionId() &&
		($msg->IsAlterAllowed() || UnbSession::IsAdministrator() || UnbSession::GetSessionUserId() == $msg->GetOwnerUserId()))
		echo '<a href="create_message.php?altmid=' . $msg->GetMessageId() . '">Alter this message</a><br />';
	echo '<a href="view_user.php?userid=' . $msg->GetOwnerUserId() . '">View user’s profile</a><br />';
	echo '</div>';

	if ($rev->GetCreatedTime() != $msg->GetCreatedTime())
	{
		echo '<div>';
		echo 'Updated <a href="list_revisions.php?messageid=' . $msg->GetMessageId() . '"><span title="' . UnbUi::DateTime($rev->GetCreatedTime(), true, true) . '">' . UnbUi::RelativeTime($rev->GetCreatedTime()) . '</span></a><br />';
		echo '</div>';
	}
	else if ($msg->CountRevisions(true) > 1)
	{
		echo '<div>';
		#echo format_ordinal($rev->GetRevisionNumber()) . ' revision updated <span title="' . UnbUi::DateTime($rev->GetCreatedTime(), true, true) . '">' . UnbUi::RelativeTime($rev->GetCreatedTime()) . '</span><br />';
		echo '<a href="list_revisions.php?messageid=' . $msg->GetMessageId() . '">List all revisions</a><br />';
		echo '</div>';
	}

	if ($msg->GetSearchRevisionId() == $rev->GetMessageRevisionId() &&
		$rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Content &&
		UnbSession::IsLoggedIn() &&
		$msg->GetOwnerUserId() != UnbSession::GetSessionUserId())
	{
		echo '<div>';
		$myRating = $msg->GetMyRating();
		if (isset($myRating))
			$myRating = round($myRating / 5.0) * 5;
		echo 'Rate: ';
		UnbUi::MsgRateLink($msg, '<a class="rate-negative' . ($myRating == -10 ? ' selected' : '') . '" href="view_message.php?%s">&minus;&minus;</a>', -10);
		UnbUi::MsgRateLink($msg, '<a class="rate-negative' . ($myRating == -5 ? ' selected' : '') . '" href="view_message.php?%s">&nbsp;&minus;&nbsp;</a>', -5);
		UnbUi::MsgRateLink($msg, '<a class="rate-positive' . ($myRating == 5 ? ' selected' : '') . '" href="view_message.php?%s">&nbsp;+&nbsp;</a>', 5);
		UnbUi::MsgRateLink($msg, '<a class="rate-positive' . ($myRating == 10 ? ' selected' : '') . '" href="view_message.php?%s">++</a>', 10);
		if (isset($myRating))
			UnbUi::MsgClearRatingLink($msg, '&nbsp; <a href="view_message.php?%s">clear</a>');
		echo '</div>';
	}

	if (UnbSession::IsModerator())
	{
		echo '<div>';
		UnbUi::MsgLockRevLink($rev, '<a href="view_message.php?%s">Lock this revision</a><br />');
		UnbUi::MsgUnlockRevLink($rev, '<a href="view_message.php?%s">Unlock this revision</a><br />');
		echo '<a href="delete_message.php?messageid=' . $msg->GetMessageId() . '">Delete this message…</a><br />';
		echo '</div>';
	}

	echo '</div>';   // .sideactions

	echo '<div class="metaline">';
	echo '<span class="permalink">';
	echo '<a href="view_message.php?messageid=' . $msg->GetMessageId() . '#message-' . $msg->GetMessageId() . '" title="Permalink">#</a>';
	echo '</span>';

	echo '<span class="author">';
	echo htmlspecialchars($msg->GetOwnerUser()->GetDisplayName());
	echo '</span>';

	$flags = $msg->GetModerationFlags();
	if ($rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject)
	{
		if (strlen($rev->GetSubject()))
		{
			if ($rev->GetSubject() != $prevSubject &&
				$rev->GetSubject() != 'Re: ' . $prevSubject)
			{
				echo '<span class="subject">“' . htmlspecialchars($rev->GetSubject()) . '”</span>';
			}
			$prevSubject = $rev->GetSubject();
		}
	}
	else
	{
		#echo '<span class="subject not-accessible">Subject not accessible</span>';
	}

	echo '<span class="date" title="' . UnbUi::DateTime($msg->GetCreatedTime(), true, true) . '">' . UnbUi::RelativeTime($msg->GetCreatedTime()) . '</span>';

	$rating = $msg->GetRating();
	if ($rating[1] >= /*30*/ 1)   // TODO: Reset for release
	{
		UnbUi::MsgRating($msg, 1, 30, '|msg-rating');
	}

	$tags = $rev->GetTags(true);
	if (!isset($prevTags))
	{
		if ($tags)
		{
			$count = 0;
			echo '<br />';
			foreach ($tags as $tag)
			{
				#if ($count++) echo ', ';
				echo '<span class="tag" title="' . htmlspecialchars($tag->GetDescription()) . '">';
				echo $tag->GetTagName();
				if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
					echo ': ' . htmlspecialchars($tag->GetTagData());
				echo '</span> ';
			}
		}
	}
	else
	{
		// Second or later revision: Show tag differences
		UnbTag::GetDifferences($prevTags, $tags, $addedTags, $removedTags, $changedTags);
		$str = '';
		foreach ($addedTags as $tag)
		{
			$str .= '<span class="tag added">+';
			$str .= $tag->GetTagName();
			if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
				$str .= ': ' . htmlspecialchars($tag->GetTagData());
			$str .= '</span> ';
		}
		foreach ($changedTags as $tag)
		{
			$str .= '<span class="tag changed">*';
			$str .= $tag->GetTagName();
			if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
				$str .= ': ' . htmlspecialchars($tag->GetTagData());
			$str .= '</span> ';
		}
		foreach ($removedTags as $tag)
		{
			$str .= '<span class="tag removed">&minus;';
			$str .= $tag->GetTagName();
			if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
				$str .= ': ' . htmlspecialchars($tag->GetTagData());
			$str .= '</span> ';
		}
		if (strlen($str))
		{
			echo '<br />' . $str;
		}
	}
	$prevTags = $tags;

	if ($rev->IsDraft())
	{
		echo '<div class="modwarning">This is a draft.</div>';
	}
	if ($msg->GetSearchRevisionId() > $rev->GetMessageRevisionId())
	{
		echo '<div class="modwarning">This is not the latest available revision of this message. <a href="view_message.php?messageid=' . $msg->GetMessageId() . '#message-' . $msg->GetMessageId() . '">View latest revision.</a></div>';
	}
	if ($flags & UnbMessageModerationFlag::Hidden)
	{
		echo '<div class="modwarning">This message has been hidden by a moderator.</div>';
	}
	if ($flags & UnbMessageModerationFlag::Locked)
	{
		echo '<div class="modwarning">This message has been locked by a moderator.</div>';
	}
	if ($rev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
	{
		echo '<div class="modwarning">This message revision is waiting for approval.';
		if (UnbSession::IsModerator())
			UnbUi::MsgApproveRevLink($rev, ' <a href="view_message.php?%s">Approve now</a><br />');
		echo '</div>';
	}
	else if ($rev->GetModerationState() == UnbMessageRevisionModerationState::Locked)
	{
		echo '<div class="modwarning">This message revision has been locked by a moderator.</div>';
	}

	$readAccessKeys = $msg->GetEffectiveReadAccessKeys();
	if (isset($readAccessKeys))
	{
		$userNames = UnbUi::GetUserList($readAccessKeys);
		echo '<div class="modwarning">Read access for: ' . str_replace("\n", ', ', $userNames) . '</div>';
	}
	echo '</div>';   // .metaline

	echo '<div class="content">';
	if ($rev->GetContentLevel() >= UnbMessageRevisionContentLevel::HtmlContent)
	{
		if ($msg->HasFileData() && preg_match('~^image/~', $rev->GetContentType()))
		{
			// Display the attached image file
			echo '<p><img src="get_data.php?messagerevisionid=' . $rev->GetMessageRevisionId() . '" alt="Attached image" style="max-width: 100%;" /></p>';
		}

		$html = $rev->GetHtmlContent();
		#$html = null;
		if (!isset($html))
		{
			if ($rev->GetContent() !== null)
			{
				$html = $markup->Convert($rev->GetContent());
				try
				{
					$rev->SetHtmlContent($html);
				}
				catch (Exception $ex) { }   // Ignore errors here
			}
			else
			{
				$html = '';   // Empty contents should always be empty HTML
			}
		}
		$html = $markup->PostProcess($html);
		if (strlen($html) > 0)
		{
			echo '<div class="markup">' . $html . '</div>';
		}
		else
		{
			if (!$msg->HasFileData())
				echo '<p><em>The message is empty.</em></p>';
			else
				echo '<p><em>This file does not have a description.</em></p>';
		}
	}
	else
	{
		echo '<div class="modwarning">Content not accessible.</div>';
	}

	if ($msg->HasFileData())
	{
		try
		{
			$size = $rev->GetFileDataLength();
			if ($rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Content)
			{
				$contentType = htmlspecialchars($rev->GetContentType());
			}
			else
			{
				$contentType = '<em>unknown</em>';
			}

			echo '<p>';
			echo '<a href="get_data.php?messagerevisionid=' . $rev->GetMessageRevisionId() . '">Download the file</a><br />';
			echo 'File type: ';
			echo '<img src="' . mimetype_icon($contentType) . '" alt="" style="vertical-align: middle; margin: -2px 5px 2px 0;" />';
			echo mimetype_description($contentType) . '<br />';
			echo 'File size: ' . format_filesize($size) . '<br />';
			echo '</p>';
		}
		catch (Exception $ex)
		{
			echo '<p style="color: red;">There is a problem with the attached data: ' . $ex->getMessage() . '</p>';
		}
	}

	$fileIds = $rev->GetFileIds();
	if (count($fileIds))
	{
		echo '<div class="message-attachments">';
		foreach ($fileIds as $fileId)
		{
			$fileMsg = UnbMessage::FindByMessageId($fileId);
			$fileRev = $fileMsg->GetSearchOrLatestRevision(UnbMessageRevisionContentLevel::Content, true);

			$size = $fileRev->GetFileDataLength();
			if ($fileRev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject)
			{
				$fileName = htmlspecialchars($fileRev->GetSubject());
				if (!strlen($fileName))
					$fileName = '<em>Unnamed file</em>';
			}
			else
			{
				$fileName = '<em>File name not accessible</em>';
			}
			if ($fileRev->GetContentLevel() >= UnbMessageRevisionContentLevel::Content)
			{
				$contentType = htmlspecialchars($fileRev->GetContentType());
			}
			else
			{
				$contentType = '<em>unknown</em>';
			}

			echo '<div class="item">';
			echo '<img src="' . mimetype_icon($contentType) . '" alt="" style="vertical-align: middle; margin: -2px 5px 2px 0;" />';
			echo '<a href="get_data.php?messageid=' . $fileId . '">' . $fileName . '</a>';
			echo ' &nbsp;–&nbsp; ' . mimetype_description($contentType) . ', ' . format_filesize($size) . ', ';
			echo '<a href="view_message.php?messageid=' . $fileId . '">View details</a>';
			if ($fileRev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
			{
				echo ' &nbsp;–&nbsp; <span class="modwarning">Waiting for approval';
				if (UnbSession::IsModerator())
					UnbUi::MsgApproveRevLink($fileRev, '. <a href="view_message.php?%s&amp;retmid=' . $msg->GetMessageId() . '">Approve now</a>');
					// TODO: Return to this message, not the file
				echo '</span>';
			}
			echo '</div>';   // .item
		}
		echo '</div>';   // .message-attachments
	}

	$flags_str = '';
	if ($flags & UnbMessageModerationFlag::EnforceApproval) $flags_str .= ' EnforceApproval';
	if ($flags_str)
		echo '<p><small>Flags: ' . trim($flags_str) . '</small></p>';
	echo '</div>';   // .content

	echo '<div style="clear: both;"></div>';
	echo '</div>';   // .message
}
?>
