-- Create database

-- TODO: Update auto value columns

-- TODO: Specify correct filename
CREATE DATABASE "unb2test" ON PRIMARY (NAME = N'unb2test', FILENAME = N'c:\Programme\Microsoft SQL Server\MSSQL.1\MSSQL\DATA\unb2test.mdf');
USE "unb2test";

-- Create base relations

CREATE TABLE "keylist" (
	"KeylistId" INTEGER NOT NULL PRIMARY KEY);

CREATE TABLE "keylist_keys" (
	"KeylistId" INTEGER NOT NULL,
	"UserId" INTEGER NOT NULL,
	PRIMARY KEY ("KeylistId", "UserId"));

CREATE TABLE "user" (
	"UserId" INTEGER NOT NULL PRIMARY KEY,
	"CreatedTime" DATETIME NOT NULL,
	"LastLoginTime" DATETIME,
	"DisplayName" VARCHAR(255),
	"DisplayNamePrep" VARCHAR(255),
	"AdditionalKeylistId" INTEGER,
	"LoginName" VARCHAR(255),
	"LoginPassword" VARCHAR(255));
CREATE UNIQUE INDEX "user_DisplayNamePrep_unq" ON "user" ("DisplayNamePrep");
CREATE UNIQUE INDEX "user_LoginName_unq" ON "user" ("LoginName");

CREATE TABLE "user_vcard" (
	"UserId" INTEGER NOT NULL,
	"Type" VARCHAR(255) NOT NULL,
	"Data" TEXT NOT NULL,
	"Visibility" TINYINT NOT NULL DEFAULT 0,
	PRIMARY KEY ("UserId", "Type"));
CREATE INDEX "user_vcard_UserId_idx" ON "user_vcard" ("UserId");

CREATE TABLE "user_config" (
	"UserId" INTEGER NOT NULL,
	"Name" VARCHAR(255) NOT NULL,
	"Data" VARCHAR(255) NOT NULL,
	PRIMARY KEY ("UserId", "Name"));
CREATE INDEX "user_config_UserId_idx" ON "user_config" ("UserId");

CREATE TABLE "message" (
	"MessageId" INTEGER NOT NULL PRIMARY KEY,
	"CreatedTime" DATETIME NOT NULL,
	"Owner" INTEGER NOT NULL,
	"PrimaryReference" INTEGER,
	"EntryPoint" BIT NOT NULL DEFAULT 0,
	"PageName" VARCHAR(255),
	"Draft" BIT NOT NULL DEFAULT 0,
	"ReadAccessKeylistId" INTEGER,
	"AlterAccessKeylistId" INTEGER,
	"ReplyAccessKeylistId" INTEGER,
	"ModerationFlags" TINYINT NOT NULL DEFAULT 0,
	"FileType" TINYINT NOT NULL DEFAULT 0,
	"SearchOptimisedContent" MEDIUMTEXT,
	"SearchOptimisedSubject" VARCHAR(255),
	"SearchRevisionId" INTEGER,
	"LastBeginEdit" DATETIME);
CREATE UNIQUE INDEX "message_PageName_unq" ON "message" ("PageName");

CREATE TABLE "message_reference" (
	"MessageId" INTEGER NOT NULL,
	"Reference" INTEGER NOT NULL,
	PRIMARY KEY ("MessageId", "Reference"));

CREATE TABLE "message_revision" (
	"MessageRevisionId" INTEGER NOT NULL PRIMARY KEY,
	"MessageId" INTEGER NOT NULL,
	"CreatedTime" DATETIME NOT NULL,
	"Author" INTEGER NOT NULL,
	"Subject" VARCHAR(255),
	"Content" MEDIUMTEXT,
	"HtmlContent" MEDIUMTEXT,
	"Summary" VARCHAR(255),
	"ModerationState" TINYINT NOT NULL DEFAULT 0,
	"Draft" BIT NOT NULL DEFAULT 0,
	"ContentType" VARCHAR(255),
	"Data" MEDIUMBLOB);
CREATE INDEX "message_revision_CreatedTime_idx" ON "message_revision" ("CreatedTime");

CREATE TABLE "tag" (
	"TagId" INTEGER NOT NULL PRIMARY KEY,
	"TagName" VARCHAR(255) NOT NULL,
	"ParentTag" INTEGER,
	"Meaning" TINYINT,
	"Description" VARCHAR(255),
	"UseAccessKeylistId" INTEGER,
	"ReadAccessKeylistId" INTEGER,
	"Flags" TINYINT NOT NULL DEFAULT 0);

CREATE TABLE "message_revision_tag" (
	"MessageRevisionId" INTEGER NOT NULL,
	"TagId" INTEGER NOT NULL,
	"TagData" VARCHAR(255),
	PRIMARY KEY ("MessageRevisionId", "TagId"));

CREATE TABLE "message_revision_file" (
	"MessageRevisionId" INTEGER NOT NULL,
	"MessageId" INTEGER NOT NULL,
	"Relation" VARCHAR(255),
	PRIMARY KEY ("MessageRevisionId", "MessageId"));

CREATE TABLE "message_rating" (
	"UserId" INTEGER NOT NULL,
	"MessageId" INTEGER NOT NULL,
	"Rating" TINYINT NOT NULL,
	"Weight" TINYINT NOT NULL,
	PRIMARY KEY ("UserId", "MessageId"));
CREATE INDEX "message_rating_MessageId_idx" ON "message_rating" ("MessageId");

CREATE TABLE "log" (
	"Num" INTEGER NOT NULL PRIMARY KEY,
	"Time" DATETIME NOT NULL,
	"Type" TINYINT NOT NULL,
	"EventId" INTEGER NOT NULL,
	"UserId" INTEGER,
	"ClientAddress" VARCHAR(100),
	"Data" VARCHAR(65535));
CREATE INDEX "log_Time_idx" ON "log" ("Time");
CREATE INDEX "log_EventId_idx" ON "log" ("EventId");

-- Create foreign keys for referential integrity
-- (MSSQL 2005 rejects some of these, so they're commented out.)

ALTER TABLE "keylist_keys" ADD FOREIGN KEY ("KeylistId") REFERENCES "keylist" ("KeylistId") ON DELETE CASCADE;
ALTER TABLE "keylist_keys" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;

ALTER TABLE "user" ADD FOREIGN KEY ("AdditionalKeylistId") REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL;

ALTER TABLE "user_vcard" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;

ALTER TABLE "user_config" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;

ALTER TABLE "message" ADD FOREIGN KEY ("Owner") REFERENCES "user" ("UserId");
-- ALTER TABLE "message" ADD FOREIGN KEY ("PrimaryReference") REFERENCES "message" ("MessageId") ON DELETE SET NULL;
ALTER TABLE "message" ADD FOREIGN KEY ("ReadAccessKeylistId") REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL;
ALTER TABLE "message" ADD FOREIGN KEY ("AlterAccessKeylistId") REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL;
ALTER TABLE "message" ADD FOREIGN KEY ("ReplyAccessKeylistId") REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL;
ALTER TABLE "message" ADD FOREIGN KEY ("SearchRevisionId") REFERENCES "message_revision" ("MessageRevisionId");

-- ALTER TABLE "message_reference" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
-- ALTER TABLE "message_reference" ADD FOREIGN KEY ("Reference") REFERENCES "message" ("MessageId") ON DELETE CASCADE;

ALTER TABLE "message_revision" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
ALTER TABLE "message_revision" ADD FOREIGN KEY ("Author") REFERENCES "user" ("UserId");

ALTER TABLE "tag" ADD FOREIGN KEY ("ParentTag") REFERENCES "tag" ("TagId");
ALTER TABLE "tag" ADD FOREIGN KEY ("UseAccessKeylistId") REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL;
ALTER TABLE "tag" ADD FOREIGN KEY ("ReadAccessKeylistId") REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL;

ALTER TABLE "message_revision_tag" ADD FOREIGN KEY ("MessageRevisionId") REFERENCES "message_revision" ("MessageRevisionId") ON DELETE CASCADE;
ALTER TABLE "message_revision_tag" ADD FOREIGN KEY ("TagId") REFERENCES "tag" ("TagId") ON DELETE CASCADE;

ALTER TABLE "message_rating" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;
ALTER TABLE "message_rating" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;

