PRAGMA foreign_keys = ON;

-- Create base relations

CREATE TABLE "keylist" (
	"KeylistId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT);

CREATE TABLE "keylist_keys" (
	"KeylistId" INTEGER NOT NULL REFERENCES "keylist" ("KeylistId") ON DELETE CASCADE,
	"UserId" INTEGER NOT NULL REFERENCES "user" ("UserId") ON DELETE CASCADE,
	PRIMARY KEY ("KeylistId", "UserId"));

CREATE TABLE "user" (
	"UserId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	"CreatedTime" DATETIME NOT NULL,
	"LastLoginTime" DATETIME,
	"DisplayName" VARCHAR(255),
	"DisplayNamePrep" VARCHAR(255),
	"AdditionalKeylistId" INTEGER REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL,
	"LoginName" VARCHAR(255),
	"LoginPassword" VARCHAR(255));
CREATE UNIQUE INDEX "user_DisplayNamePrep_unq" ON "user" ("DisplayNamePrep");
CREATE UNIQUE INDEX "user_LoginName_unq" ON "user" ("LoginName");

CREATE TABLE "user_vcard" (
	"UserId" INTEGER NOT NULL REFERENCES "user" ("UserId") ON DELETE CASCADE,
	"Type" VARCHAR(255) NOT NULL,
	"Data" TEXT NOT NULL,
	"Visibility" TINYINT NOT NULL DEFAULT 0,
	PRIMARY KEY ("UserId", "Type"));
CREATE INDEX "user_vcard_UserId_idx" ON "user_vcard" ("UserId");

CREATE TABLE "user_config" (
	"UserId" INTEGER NOT NULL REFERENCES "user" ("UserId") ON DELETE CASCADE,
	"Name" VARCHAR(255) NOT NULL,
	"Data" VARCHAR(255) NOT NULL,
	PRIMARY KEY ("UserId", "Name"));
CREATE INDEX "user_config_UserId_idx" ON "user_config" ("UserId");

CREATE TABLE "message" (
	"MessageId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	"CreatedTime" DATETIME NOT NULL,
	"Owner" INTEGER NOT NULL REFERENCES "user" ("UserId"),
	"PrimaryReference" INTEGER REFERENCES "message" ("MessageId") ON DELETE SET NULL,
	"EntryPoint" BOOLEAN NOT NULL DEFAULT FALSE,
	"PageName" VARCHAR(255),
	"Draft" BOOLEAN NOT NULL DEFAULT FALSE,
	"ReadAccessKeylistId" INTEGER REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL,
	"AlterAccessKeylistId" INTEGER REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL,
	"ReplyAccessKeylistId" INTEGER REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL,
	"ModerationFlags" TINYINT NOT NULL DEFAULT 0,
	"FileType" TINYINT NOT NULL DEFAULT 0,
	"SearchOptimisedContent" MEDIUMTEXT,
	"SearchOptimisedSubject" VARCHAR(255),
	"SearchRevisionId" INTEGER REFERENCES "message_revision" ("MessageRevisionId"),
	"LastBeginEdit" DATETIME);
CREATE UNIQUE INDEX "message_PageName_unq" ON "message" ("PageName");

CREATE TABLE "message_reference" (
	"MessageId" INTEGER NOT NULL REFERENCES "message" ("MessageId") ON DELETE CASCADE,
	"Reference" INTEGER NOT NULL REFERENCES "message" ("MessageId") ON DELETE CASCADE,
	PRIMARY KEY ("MessageId", "Reference"));

CREATE TABLE "message_revision" (
	"MessageRevisionId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	"MessageId" INTEGER NOT NULL REFERENCES "message" ("MessageId") ON DELETE CASCADE,
	"CreatedTime" DATETIME NOT NULL,
	"Author" INTEGER NOT NULL REFERENCES "user" ("UserId"),
	"Subject" VARCHAR(255),
	"Content" MEDIUMTEXT,
	"HtmlContent" MEDIUMTEXT,
	"Summary" VARCHAR(255),
	"ModerationState" TINYINT NOT NULL DEFAULT 0,
	"Draft" BOOLEAN NOT NULL DEFAULT FALSE,
	"ContentType" VARCHAR(255),
	"Data" BLOB);
CREATE INDEX "message_revision_CreatedTime_idx" ON "message_revision" ("CreatedTime");

CREATE TABLE "tag" (
	"TagId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	"TagName" VARCHAR(255) NOT NULL,
	"ParentTag" INTEGER REFERENCES "tag" ("TagId"),
	"Meaning" TINYINT,
	"Description" VARCHAR(255),
	"UseAccessKeylistId" INTEGER REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL,
	"ReadAccessKeylistId" INTEGER REFERENCES "keylist" ("KeylistId") ON DELETE SET NULL,
	"Flags" TINYINT NOT NULL DEFAULT 0);

CREATE TABLE "message_revision_tag" (
	"MessageRevisionId" INTEGER NOT NULL REFERENCES "message_revision" ("MessageRevisionId") ON DELETE CASCADE,
	"TagId" INTEGER NOT NULL REFERENCES "tag" ("TagId") ON DELETE CASCADE,
	"TagData" VARCHAR(255),
	PRIMARY KEY ("MessageRevisionId", "TagId"));

CREATE TABLE "message_revision_file" (
	"MessageRevisionId" INTEGER NOT NULL,
	"MessageId" INTEGER NOT NULL,
	"Relation" VARCHAR(255),
	PRIMARY KEY ("MessageRevisionId", "MessageId"));

CREATE TABLE "message_rating" (
	"UserId" INTEGER NOT NULL REFERENCES "user" ("UserId") ON DELETE CASCADE,
	"MessageId" INTEGER NOT NULL REFERENCES "message" ("MessageId") ON DELETE CASCADE,
	"Rating" TINYINT NOT NULL,
	"Weight" TINYINT NOT NULL,
	PRIMARY KEY ("UserId", "MessageId"));
CREATE INDEX "message_rating_MessageId_idx" ON "message_rating" ("MessageId");

-- Create triggers

CREATE TRIGGER "keylist_empty_trg"
AFTER DELETE ON "keylist_keys" FOR EACH ROW BEGIN
	DELETE FROM "keylist"
	WHERE "KeylistId" = OLD."KeylistId"
		AND NOT EXISTS (SELECT "KeylistId" FROM "keylist_keys" WHERE "KeylistId" = OLD."KeylistId");
END;
