<?php
// UNB Components
// Copyright since 2007 by Yves Goergen
// Website: http://unclassified.de/go/unb2
// Licence: GNU General Public License (GPL) Version 3
//
// Database setup script

// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
header('Content-Type: text/html; charset=utf-8');

// Retain output until after the session is initialised
ob_start();

echo '<p><big>UNB Components database setup script</big></p>';

echo 'Including library and class files...<br>';
#require_once('unb_lib/unb_all.lib.php');
if (!class_exists('UnbEnvironment')) require_once('unb_lib/environment.class.php');
if (!class_exists('UnbDatabase')) require_once('unb_lib/database.class.php');
if (!class_exists('UnbUser')) require_once('unb_lib/user.class.php');
if (!class_exists('UnbMessage')) require_once('unb_lib/message.class.php');
if (!class_exists('UnbTag')) require_once('unb_lib/tag.class.php');
if (!class_exists('UnbSession')) require_once('unb_lib/session.class.php');

UnbConfiguration::SetFilename('../unb_config.inc.php');

$db = UnbDatabase::GetInstance();

echo 'Connecting to database...<br>';
$db->Open();

echo 'Clearing all stored data...<br>';
$db->StartTransaction();
$db->Execute('DELETE FROM "#keylist"');
$db->Execute('UPDATE "#tag" SET "ParentTag" = NULL');
$db->Execute('DELETE FROM "#tag"');
$db->Execute('UPDATE "#message" SET "SearchRevisionId" = NULL');
$db->Execute('DELETE FROM "#message"');
$db->Execute('DELETE FROM "#user"');
try
{
	$db->Execute('DELETE FROM "#log"');
}
catch (UnbDatabaseException $ex)
{
	echo '<strong>Warning:</strong> Log table could not be cleared.<br>';
}
$db->Commit();

echo 'Starting session...<br>';
UnbSession::EnableSession();

echo 'Creating system role keys...<br>';
$user_system = UnbUser::Create('System key', null, null, UnbUserKey::System);
$user_admin = UnbUser::Create('Administrator key', null, null, UnbUserKey::Administrator);
$user_mod = UnbUser::Create('Moderator key', null, null, UnbUserKey::Moderator);
$user_trusted = UnbUser::Create('Trusted user key', null, null, UnbUserKey::Trusted);
$user_default = UnbUser::Create('Default user key', null, null, UnbUserKey::DefaultUser);

echo 'Logging in as admin user...<br>';
// Setup some things as administrator
UnbSession::SetSessionUser($user_admin, null, true);

echo 'Creating the first user...<br>';
$user_first = UnbUser::Create('First user', 'first', 'first');
$user_first->AddAdditionalKey($user_mod);
$user_first->AddAdditionalKey($user_admin);
$user_first->AddAdditionalKey($user_trusted);

echo 'Logging in as first user...<br>';
// Now work on with a regular user
UnbSession::SetSessionUser($user_first, 'first');

echo 'Setting up first user\'s vcard...<br>';
$user_first->SetVcardEntry('RealName', 'First User', UnbUserVcardVisibility::Public_);

echo 'Logging out...<br>';
// Log out to clear auto-login cookie which may be set in the wrong URL path here
UnbSession::SetSessionUser(null);

echo '<p><strong>Finished</strong><br>You can now log in with the name "first" and password "first".</p>';
?>
